# -*- coding: utf-8 -*-

import sys, json, time, struct, datetime
from os.path import dirname, abspath
from google.protobuf import json_format
project_path = dirname(dirname(abspath(__file__)))
projec_full_path = project_path + r'\py_protoc'
sys.path.append(projec_full_path)
from pub_cls_com import *
from com import *
############################旧版################################
# from py_protoc import msg_stock_pb2 as msg_stock
# from py_protoc import msg_stock_pb2 as msg_stock
# from py_protoc import msg_stock_common_pb2 as msg_stock_common
############################旧版################################

from py_protoc import msg_comm_request_pb2 as msg_stock_common
from py_protoc import msg_comm_cust_pb2 as cust_pb
from py_protoc import msg_comm_response_pb2 as reponse_pb
from py_protoc import stock_comm_pb2 as comm_pb
from py_protoc import stock_real_pb2 as msg_stock    # real_pb

class KaisaProtoc(object):

    def __init__(self):
        pass

    @staticmethod
    def tcp_login(token, macip, ip, userid, reqid):
        # tcp 登录、初始化行情api
        pb_login = comm_pb.Login()
        pb_login.token = token
        pb_login.deviceId = macip
        pb_login.macAddress = macip
        pb_login.ipAddress = ip
        pb_login.sysVersion = SYSVERSION
        pb_login.platform = PLATFORM
        pb_login.phoneModel = PHONEMODEL
        pb_login.srvType = SRVTYPE
        pb_login.username = EP
        pb_login.password = EP
        pb_login.userId = userid
        pb_string = pb_login.SerializeToString()
        return KaisaProtoc.get_reqhead(20, 0, pb_string, reqid)

    @staticmethod
    def get_head_bstring():
        #  通讯头
        packFlag = struct.pack('>h', 2021)
        iCompress = struct.pack('>h', 0)
        iVersion = struct.pack('>h', 0)
        iReserve = struct.pack('>i', 0)
        iCheckId = struct.pack('>h', 1990)
        head_bstrin = packFlag + iCompress + iVersion + iReserve + iCheckId
        return head_bstrin

    @staticmethod
    def get_cust_head_bstring():
        #cust_head
        cust_c = cust_pb.CustHead()
        cust_c.packFlag = 2021
        cust_c.iCompress = 0
        cust_c.iVersion = 0
        cust_c.iReserve = 0
        cust_c.iCheckId = 1990
        return cust_c

    @staticmethod
    def get_reqhead(mod, cmd, data, reqid=None):
        # 请求头
        #len_data = len(data)
        if reqid is not None:
            reqId = struct.pack('>i', reqid)
        else:
            reqId = 0
        # 心跳
        if mod == 3:
            data = b'0'
        mod = struct.pack('>h', mod)
        cmd = struct.pack('>h', cmd)
        datalen = struct.pack('>i', len(data))
        #len_custhead = len(KaisaProtoc.get_head_bstring())
        reqhead = KaisaProtoc.get_head_bstring() + reqId + mod + cmd + datalen
        #len_reqhead = len(reqhead)
        result = reqhead + data
        #len_rresult = len(result)
        return result
    @staticmethod
    def get_reshead(data):
        #
        res = KaisaProtoc.get_head_bstring() + data
        return res

    @staticmethod
    def get_ping_bstring(reqId=1):
        # 心跳
        pb_string = KaisaProtoc.get_reqhead(3, 0, b'0', reqId)
        return pb_string

    @staticmethod
    def get_subscribe_bstring(symbollist, reqId=1):
        '''
        订阅行情
        SUB_TYPE_NONE = 0;
        QUOTE = 1;
        TICK = 2;
        BROKER = 3;
        ORDER = 4;
        '''
        pb_rule = msg_stock.Rule()
        #pb_stockcode = msg_stock.StockCode()
        for secucode_dict in symbollist:
            pb_stockcode_one = pb_rule.stockCodes.add()
            pb_stockcode_one.code = secucode_dict['code']
            market = secucode_dict['market']
            pb_stockcode_one.market = market
            pb_stockcode_one.language = 0
            pb_stockcode_one.types.append(secucode_dict['sub_type'])

        ps = pb_rule.SerializeToString()
        mod = 1
        cmd = 10
        pb_string = KaisaProtoc.get_reqhead(mod, cmd, ps, reqId)
        return pb_string

    @staticmethod
    def get_unsubscribe_bstring(symbollist, reqId=1):
        '''
        退订行情
        SUB_TYPE_NONE = 0;
        QUOTE = 1;
        TICK = 2;
        BROKER = 3;
        ORDER = 4;
        '''
        mod = 1
        cmd = 11
        pb_rule = msg_stock.Rule()
        pb_stockcode = msg_stock.StockCode()
        for secucode_dict in symbollist:
            pb_stockcode_one = pb_rule.stockCodes.add()
            pb_stockcode_one.code = secucode_dict['code']
            market = secucode_dict['market']
            pb_stockcode_one.market = market
            pb_stockcode_one.language = 0
            pb_stockcode_one.types.append(secucode_dict['sub_type'])
        ps = pb_rule.SerializeToString()

        pb_string = KaisaProtoc.get_reqhead(mod, cmd, ps, reqId)
        return pb_string

    @staticmethod
    def rec_bstring_to_json(bstring):
        '''
        uint32 cmd = 4; //对应模块编号下的指令 2位
        {QUOTE = 1;
        TICK = 2;
        }
        bytes data = 7; //二进制数据 4位

        # '!'表示我们用网络字节序解析,因为我们是从网络上接收到这个buf的。
        # 'H'表示unsigned short的id；
        # '4s'表示四个字节长的字符串； char[4]
        # '2I'表示有两个unsigned int类型的数据；
        '''

        data_type = 0
        try:
            if len(bstring) >= 28:
                cmd, __ = get_respdata(bstring)
                if __ == '' or cmd not in ALLDATAIDLIST:
                    return {'type': 0, 'data': None}
                else:
                    data_type, res_data = KaisaProtoc.handle_any_data_cls(cmd, __)
                    return {'type': data_type, 'data': res_data}
            else:
                return {'type': 0, 'data': None}
        except BaseException as exp:
            print('获取行情抛异常:{}'.format(exp))
            return {'type': data_type, 'data': None}


    @staticmethod
    def quote_bstring_to_json(quote):
        ## 解析quote-data
        sec = quote.quote_time.seconds
        nano = quote.quote_time.nanos
        # up_t = datetime.datetime.fromtimestamp((sec + nano / 1000000000.0))
        update_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(sec + nano / 1000000000.0))
        return {'market': quote.market,
                'code': quote.code,
                'now': quote.now,
                'high': quote.high,
                'low': quote.low,
                'volume': quote.volume,
                'amount': quote.amount,
                'quote_time': update_time,
                }

    @staticmethod
    def broker_list_bstring_to_json(broker_list):
        # broker-list data to json

        return {'market': broker_list.market,
                'code': broker_list.code,
                'time': time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(
                    broker_list.time.seconds+broker_list.time.nanos/1000000000.0)),
                'brokerSell': [KaisaProtoc.broker_bstring_to_json(bs) for bs in broker_list.brokerSell],
                'brokerBuy': [KaisaProtoc.broker_bstring_to_json(bb) for bb in broker_list.brokerBuy]
                }

    @staticmethod
    def broker_bstring_to_json(broker):
        ## broker data to json
        return {'broker_level': broker.broker_level,
                'broker_id': [KaisaProtoc.broker_id_bstring_to_json(_id_) for _id_ in broker.broker_id]
                }

    @staticmethod
    def broker_id_bstring_to_json(broker):
        # broker-is data to json
        return {'broker_id': broker}

    @staticmethod
    def ticklist_bstring_to_json(ticklist):

        return {'market': ticklist.market,
                'code': ticklist.code,
                'tick': [KaisaProtoc.tick_bstring_to_json(item) for item in ticklist.tick]
                }

    @staticmethod
    def tick_bstring_to_json(tick):

        return {'now': tick.now,
                'cur_volume': tick.cur_volume,
                'tick_flag': tick.tick_flag,
                'tick_vi': tick.tick_vi,
                'tick_time': time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(
                    tick.tick_time.seconds+tick.tick_time.nanos/1000000000.0))
                }

    @staticmethod
    def level_order_list_string_to_json(order_list):
        # level_order list data

        return {'market': order_list.market,
                'code': order_list.code,
                'time': time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(
                    order_list.time.seconds+order_list.time.nanos/1000000000.0)),
                'orderSell': [KaisaProtoc.level_order_bsstring_to_json(bs) for bs in order_list.orderSell],
                'orderBuy': [KaisaProtoc.level_order_bsstring_to_json(bb) for bb in order_list.orderBuy]
                }
    @staticmethod
    def level_order_bsstring_to_json(order_bs):
        # 转换order中的bs数据
        return {'level': order_bs.level,
                'price': order_bs.price,
                'volume': order_bs.volume,
                'broker_count': order_bs.broker_count
                }

    @staticmethod
    def handle_any_data_cls(DATATYPE, data):
        # total-handle-data
        try:
            THISDATATYPE = PROTODATADIC[DATATYPE]
            if THISDATATYPE == 'QUOTE':
                return KaisaProtoc._get_quote_data(data)
            elif THISDATATYPE == 'TICK':
                return KaisaProtoc._get_tick_data(data)
            elif THISDATATYPE == 'BROKER':
                return KaisaProtoc._get_broker_data(data)
            elif THISDATATYPE == 'ORDER':
                return KaisaProtoc._get_level_order_data(data)
            else:
                pass
        except Exception as exp:
            print('get-quote-data-_dic_ error:{}'.format(exp))
            return None

    @staticmethod
    def _get_tick_data(data):
        #### cmd=2
        #### 逐笔数据 tick

        body_json = KaisaProtoc.ticklist_bstring_to_json(data)
        return PROTODATADIC[TICK], body_json

    @staticmethod
    def _get_quote_data(data):
        #### cmd=1
        #### 盘口数据 quote

        try:
            ## 判断该条行情是否有效
            if data.update_flag != [1, 4, 8, 16, 32, 2048, 32768]:
                return QUOTE, None
            else:
                body_json = KaisaProtoc.quote_bstring_to_json(data)
                return PROTODATADIC[QUOTE], body_json
        except Exception as exp:
            print('get-quote-data-error:{}'.format(exp))

    @staticmethod
    def _get_broker_data(data):
        #### cmd=3
        #### 经纪商数据 broker-list
        body_json = KaisaProtoc.broker_list_bstring_to_json(data)
        return PROTODATADIC[BROKERS], body_json

    @staticmethod
    def _get_level_order_data(data):
        ### cmd=4
        ### level-10 order data
        body_json = KaisaProtoc.level_order_list_string_to_json(data)
        return PROTODATADIC[LEVEL_ORDER], body_json

if __name__=='__main__':
    print('start.')




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg_comm_request.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import msg_comm_cust_pb2 as msg__comm__cust__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg_comm_request.proto',
  package='MsgCommRequest',
  syntax='proto3',
  serialized_options=b'\n\034com.kaisaglobal.proto.common\200\001\000\210\001\001\220\001\000',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16msg_comm_request.proto\x12\x0eMsgCommRequest\x1a\x13msg_comm_cust.proto\"v\n\x07ReqHead\x12\'\n\x08\x63ustHead\x18\x01 \x01(\x0b\x32\x15.MsgCommCust.CustHead\x12\r\n\x05reqId\x18\x02 \x01(\r\x12\x0b\n\x03mod\x18\x03 \x01(\r\x12\x0b\n\x03\x63md\x18\x04 \x01(\r\x12\x0b\n\x03len\x18\x05 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x42\'\n\x1c\x63om.kaisaglobal.proto.common\x80\x01\x00\x88\x01\x01\x90\x01\x00\x62\x06proto3'
  ,
  dependencies=[msg__comm__cust__pb2.DESCRIPTOR,])




_REQHEAD = _descriptor.Descriptor(
  name='ReqHead',
  full_name='MsgCommRequest.ReqHead',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='custHead', full_name='MsgCommRequest.ReqHead.custHead', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reqId', full_name='MsgCommRequest.ReqHead.reqId', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mod', full_name='MsgCommRequest.ReqHead.mod', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='MsgCommRequest.ReqHead.cmd', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='len', full_name='MsgCommRequest.ReqHead.len', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MsgCommRequest.ReqHead.data', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=181,
)

_REQHEAD.fields_by_name['custHead'].message_type = msg__comm__cust__pb2._CUSTHEAD
DESCRIPTOR.message_types_by_name['ReqHead'] = _REQHEAD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ReqHead = _reflection.GeneratedProtocolMessageType('ReqHead', (_message.Message,), {
  'DESCRIPTOR' : _REQHEAD,
  '__module__' : 'msg_comm_request_pb2'
  # @@protoc_insertion_point(class_scope:MsgCommRequest.ReqHead)
  })
_sym_db.RegisterMessage(ReqHead)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)




# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# FinanceData
# ===================================================================
import sys
from os.path import dirname, abspath
import requests, datetime
import pandas as pd
import numpy as np
import json
from common.com import *
from pub_cls_com import *
'''Date: 2021.03.15'''

################################财务数据##################################
###### 基础财务数据(主要财务数据、三大报表)
def _handle_req_ret(period, report_type, code, market):
    # 基础财务数据: 公共统一处理请求和返回数据
    # 市场类型: 港股：hkg，美股：usa, a股：chn
    # 市场类型：1 CH/ 2 HK/ 3 USA
    this_market = FIN_MARKET_TYPE_DICT[market]
    if t_typeof(code) != 'str':
        return FIN_NOE_CODE_ERROR
    if market == 'usa' or market == 'chn':
        return ONLY_HK_MARKET_ERROR
    if code is None:return FIN_LOSS_CODE_ERROR
    body = {
        "reportType": report_type,
        "periodMark": period,
        "stockCode": code,
        "stockMarket": this_market
    }
    update_url = lambda url: url.replace("__kaisarooturl__", KAISA_ROOT_URL)
    _df_ = total_handle_req(body=body, URL=update_url(BASIC_FIN_POST_URL))
    return _df_

def get_basic_fin_data(code='00700', market=2, period=12):
    '''
    periodMark	integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
    reportType	string 1：主要指标
    stockCode	string 股票代码
    stockMarket	string 港股：hkg，美股：usa,a股：chn
    Returns:
        df:
        主要财务指标  季度/年度的财务数据
    '''
    report_type = '1'
    basic_fin_df = _handle_req_ret(period, report_type, code, market)

    return basic_fin_df

def get_income_fin_data(code='00700', market=2, period=12):
    '''
    Args:
        code: string 股票代码
        market: string 港股：hkg，美股：usa,a股：chn
        period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
        report_type: string 2：利润表
    Returns:
        df:
        income 季度/年度的财务数据
    '''
    report_type = '2'
    income_fin_df = _handle_req_ret(period, report_type, code, market)

    return income_fin_df

def get_balance_fin_data(code='00700', market=2, period=12):
    '''
    Args:
        code: string 股票代码
        market: string 港股：hkg，美股：usa,a股：chn
        period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
        report_type: string 3：资产负债表
    Returns:
        df:
        balance 季度/年度的财务数据
    '''
    report_type = '3'
    balance_fin_df = _handle_req_ret(period, report_type, code, market)

    return balance_fin_df

def get_cashflow_fin_data(code='00700', market=2, period=12):
    '''
    Args:
        code: string 股票代码
        market: string 港股：hkg，美股：usa,a股：chn
        period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
        report_type: string 4：现金流量表
    Returns:
        df
        cashflow  季度/年度的财务数据
    '''
    report_type = '4'
    cashflow_fin_df = _handle_req_ret(period, report_type, code, market)

    return cashflow_fin_df

###### 衍生财务数据(财务因子)

def _handle_ex_ret(code_list=['00700'], queryType=None, start_date=None, end_date=None, market=2, period=12, page=1):
    #

    if market == 3 or market == 1:
        return ONLY_HK_MARKET_ERROR
    if start_date is None and end_date is None:
        return PUB_DATENONE_ERROR
    if start_date is not None and end_date is not None:
        if start_date > end_date:
            return PUB_SMALLENDDATE_ERROR
    if start_date is not None and end_date is None:
        end_date = start_date
    if start_date is None and end_date is not None:
        start_date = end_date
    if queryType == BASE_FACE:
        body = {
            "codeList": code_list,
            "market": market,
            "queryType": queryType,
            "startDate": start_date,
            "endDate": end_date,
            "periodMark": period,
            "current": page
        }
    else:
        body = {
            "codeList": code_list,
            "market": market,
            "queryType": queryType,
            "periodMark": period,
            "current": page
        }
    update_url = lambda url: url.replace("__kaisarooturl__", KAISA_ROOT_URL)
    df_ = total_handle_req(body=body, URL=update_url(EX_FIN_POST_URL))

    return df_

def _handle_get_fina_data_cls(_data_, data_type, code_list, start_date, end_date, market, period):
    # 统一get财务数据
    try:
        info_date = 'info_publ_date'
        date_type_dic = {BASE_FACE: 'trading_day', PROFIT: info_date,
                         CASHFLOW: info_date, OPERATION: info_date, DEBTPAY: info_date, GROWTH: info_date}

        dt_filter_field = date_type_dic[data_type]

        pageSize = _data_['totalPage']
        page1_data = _data_['result']
        df1_ = pd.DataFrame(page1_data)
        for pageC in range(2, pageSize + 1):
            profit_data_next = _handle_ex_ret(code_list, data_type, start_date, end_date, market, period, pageC)
            page_n_data = profit_data_next['result']
            df1_ = pd.concat([df1_, pd.DataFrame(page_n_data)], axis=0)

        df1_ = df1_.drop_duplicates(subset=['secu_code', dt_filter_field])   # 去重
        if start_date is not None and end_date is not None:
            df1_ = df1_[(df1_[dt_filter_field] >= start_date) & (df1_[dt_filter_field] <= end_date)]
            ch_sign = 2
        else:
            cur_dt = end_date if start_date is None else start_date
            ch_sign = 1
            df1_ = df1_[df1_[dt_filter_field] <= cur_dt]


        __ = _get___(df1_, ch_sign, dt_filter_field)
        __ = __.reset_index(drop=True)
    except Exception as exp:
        return PUB_WRONG_MSG_ERROR
    return __

def _get___(df, ch_sign=None, dt_filter_field='info_publ_date'):
    '''
    Args:
        df: 原始数据
        dt_type: 时间类型【start_date/end_date】
    Returns:
    返回所需的衍生财务/基本面因子数据
    '''
    # 统一过滤得到最后需要的财务数据-df
    __ = pd.DataFrame()
    gp_df = df.groupby(by=['secu_code'])
    for gp, ___ in gp_df:
        if ch_sign == 1:
            ___ = ___.sort_values(by=[dt_filter_field], ascending=False)
            ___ = ___.head(1)
        if __.empty:
            __ = ___.copy()
        else:
            __ = pd.concat([__, ___], axis=0)
    return __

def get_profit_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取盈利能力因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是上市公司公布财报的日期)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    __ = _all_fina_factor_handle_method(code_list, PROFIT, start_date, end_date, market, period)
    return __

def get_cash_flow_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取现金流因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是上市公司公布财报的日期)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    data_type = 'cashFlow'
    __ = _all_fina_factor_handle_method(code_list, CASHFLOW, start_date, end_date, market, period)
    return __

def get_operation_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取运营能力因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是上市公司公布财报的日期)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    data_type = 'operation'
    __ = _all_fina_factor_handle_method(code_list, OPERATION, start_date, end_date, market, period)
    return __

def get_growth_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取成长能力因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是上市公司公布财报的日期)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    data_type = 'growth'
    __ = _all_fina_factor_handle_method(code_list, GROWTH, start_date, end_date, market, period)
    return __

def get_debtpay_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取偿债能力因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是上市公司公布财报的日期)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    data_type = 'debtPay'
    __ = _all_fina_factor_handle_method(code_list, DEBTPAY, start_date, end_date, market, period)
    return __

def get_baseface_factor_fin_data(code_list=['00700'], start_date=None, end_date=None, market=2, period=12):
    '''
        获取一般基本面因子数据
        Args:
            code_list: 股票篮子 ['00700']/['00700','06969']
            start_date: 开始日期 如 ‘2016-12-31’
            end_date: 结束日期 如 ‘2018-12-31’
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            period: 报表类型：0全部, 3一季报,6中报,9三季报,12年报
        Returns:
        PS： 1.如果指定了start_date和end_date那返回的是这个区间的财务因子数据,(时间对应的是每日行情数据更新后的交易日)
            2.如果只指定了开始日期或者结束日期则取这个日期的最近一期的财务因子数据
            3.如果两个日期都没指定，则取最近一期的财务因子数据
        对应股票篮子的历史财务数据
    '''
    data_type = 'baseFace'
    if t_typeof(code_list) != 'list': return pd.DataFrame()
    if end_date is None and start_date is None:
        return PUB_DATENONE_ERROR

    if end_date is None and start_date is not None:
        end_date = start_date
    if end_date is not None and start_date is None:
        start_date = end_date
    __ = _all_fina_factor_handle_method(code_list, BASE_FACE, start_date, end_date, market, period)
    return __

def _all_fina_factor_handle_method(code_list=['00700'], data_type=None, start_date=None, end_date=None, market=2, period=12):
    # 集中处理_handle_ex_ret/_handle_get_fina_data_cls
    code_list = list(set(code_list))
    if t_typeof(code_list) != 'list':
        return PUB_ONLY_LIST_TYPE_ERROR
    _data_ = _handle_ex_ret(code_list, data_type, start_date, end_date, market, period)
    try:
        if PUB_HELLO in _data_ or PUB_SOR in _data_:
            return _data_
        if len(_data_['result']) == 0:
            return PUB_WRONG_DATE_AROUND_ERROR
        __ = _handle_get_fina_data_cls(_data_, data_type, code_list, start_date, end_date, market, period)

        return __
    except:
        return _data_


################################财务数据##################################



# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# ===================================================================

"""
This code provide basic market and trade function for KaisaGlobal-Open.
Pls. note that only HK market is supported now.

developed by KaisaGlobal quant team.
2020.12.11
"""


import os, sys, json

import requests
import threading

import websocket
from copy import deepcopy
from pub_cls_com import *

from os.path import dirname, abspath
import socket

project_path = dirname(dirname(abspath(__file__)))
projec_full_path = project_path + r'\common'
sys.path.append(projec_full_path)

from com import *
from KaisaCrypto import *
from utils import *
from KaisaProtoc import *

kcrypto = KaisaCrypto()
kprotoc = KaisaProtoc()

class KaisaGateway(object):

    req_id = 0
    # gateway - market, trade;
    gateway_auth_status = False
    # ws
    market_ws_alive = False
    trade_ws_alive = False
    # trade;
    trade_connect_status = False
    trade_auth_status = False
    # token
    gatewayToken = None
    tradeToken = None
    _market_ws = None
    _trade_ws = None

    _market_connect_status = False
    _ws_trade_connect_status = False
    _market_connect_msg = ""

    def __init__(self):
        pass

    def jq_user_config(self, user_id,
                       user_pwd,
                       account_code,
                       account_pwd,
                       openapi_token,
                       ip_address='',
                       callback=None):
        # 构造
        self.user_id = user_id
        self.user_pwd = user_pwd
        self.account_code = account_code
        self.account_pwd = account_pwd
        self.openapi_token = openapi_token
        self.root_url = KAISA_ROOT_URL   # KAISA_ROOT_URL_SIM if self.environment in ["simulate", "paper"] else
        self._update_urls()
        self.init_api_bool = False
        self.ip_address = ip_address
        # AF_INET:ipv4, SOCK_STREAM:tcp
        self._open_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.subscribe_code_list = []
        self.callback_cls = callback
        self.userid = ''
        self.gatewayToken = ''
        self.gateway_auth_status = False
        self.gateway_auth_msg = ''
        self.th = None
        self.trade_access_token = None
        self.file_name = project_path + '/logger/heatlog.txt'

    def init_open_md_api(self):
        # 初始化行情api--TCP>>Login

        if self.gateway_auth_status:
            return self.gateway_auth_status, self.gateway_auth_msg

        auth_data = self.gateway_auth()
        use_id = auth_data['userid']
        token = auth_data['token']
        ret_ok = auth_data['status']
        print(f'md_api-开始连接, 更新时间:{datetime.datetime.now()}')
        # 请求登录和初始化行情api
        self._open_sock.connect((OPENIP, OPENPORT))
        self.req_id += 1
        bs_param = KaisaProtoc.tcp_login(token,
                                         get_mac_addr(),
                                         self.ip_address,
                                         use_id,
                                         self.req_id)
        self._open_sock.sendall(bs_param)
        # 获取服务端返回的数据
        res_data = self.on_recv()
        print(f'md_api-连接状态:{res_data}')

        time.sleep(1)

    def _update_urls(self):

        update_url = lambda url: url.replace("__kaisarooturl__", self.root_url)

        self.CRYPTO_RSA_URL = update_url(CRYPTO_RSA_URL)
        self.AUTHENTICA_URL = update_url(AUTHENTICA_URL)
        self.QUOTE_URL = update_url(QUOTE_URL)

        self.REST_DATA_HOST = update_url(REST_DATA_HOST)
        #self.WEBSOCKET_DATA_HOST = update_url(WEBSOCKET_DATA_HOST)

        self.ClientByMobile_URL = update_url(ClientByMobile_URL)
        self.REST_HOST = update_url(REST_HOST)
        self.WEBSOCKET_TRADE_HOST = update_url(WEBSOCKET_TRADE_HOST)

        self.TRADE_CRYPTO_RSA_URL = update_url(TRADE_CRYPTO_RSA_URL)

    def write_error(self, data):
        print(f"error: {data}")

    def write_log(self, data):
        print(f"log: {data}")

    def _authentica(self, auth_username: str, auth_password: str) -> None:
        """
        ## sockt 登录
        获取网关登录令牌;
        :param auth_username:
        :param auth_password:
        :return:
        """

        if self.gateway_auth_status:
            return {'status': self.gateway_auth_status, 'msg': self.gateway_auth_msg, 'token': self.gatewayToken, 'userid': self.userid}
        gateway_auth_msg = ""

        try:
            # only for token of gateway.
            timestamp_ = generate_timestamp()
            sign_ = kcrypto.encrypt_md5("username{}Timestamp{}".format(auth_username, timestamp_))

            auth_username_encrypt = kcrypto.encrypt_rsa_username(auth_username, crypto_rsa_url=self.CRYPTO_RSA_URL)
            auth_password_encrypt = kcrypto.encrypt_aes_password(auth_password, "MAKRET")

            params = {
                "username": auth_username_encrypt,
                "password": auth_password_encrypt,
                "grant_type": "password",
                "scope": openapi_scope
            }

            ipAddress = self.ip_address
            if ipAddress:
                params["ipAddress"] = ipAddress

            headerParam = {"srvType": "open", "ipAddress": ipAddress, "phoneModel": "",
                           "deviceId": "", "deviceToken": "", "macAddress": get_mac_addr(),
                           "channelId": "", "country": "", "sysVersion": "",
                           "apiVersion": "", "platform": ""}
            header_userid_param = {"platform": "web"}
            headerParam = json.dumps(headerParam)
            header_userid_param = json.dumps(header_userid_param)
            authorization_str = "basic {}".format(self.openapi_token)
            headers = {
                "Authorization": authorization_str,
                "Content-Type": "application/x-www-form-urlencoded",
                "Sign": sign_,
                "Timestamp": timestamp_,
                "headerParam": headerParam
            }

            response = requests.post(url=self.AUTHENTICA_URL, params=params, headers=headers)

            data = response.json()
            if response.status_code // 100 == 2:
                # get_userid-url
                self.write_log("网关认证成功")
                if data['success']:
                    self.write_log("获取登录令牌成功")
                    trade_token_body = data["body"]["accessToken"]   #  data['boday']['accessToken']
                    self.trade_access_token = f"bearer {trade_token_body}"
                    __, ___ = self._get_accessToken(data)
                    __gatewayToken__ = __
                    gateway_auth_status = ___

                    __userid__ = self._get_user_info(__, header_userid_param)
                else:
                    self.token = None
                    self.write_log("获取登录令牌失败")
                    self.write_error(data)
                    gateway_auth_status = False
                    gateway_auth_msg = None
                    __userid__ = None
                    __gatewayToken__ = None
            else:
                self.write_log("网关认证失败")
                self.write_error(data)
                gateway_auth_status = False
                gateway_auth_msg = None
                __userid__ = None
                __gatewayToken__ = None
            # self.gatewayToken = f"bearer {token_body}"
            # self.token = self.gatewayToken
            self.gatewayToken = __gatewayToken__
            self.gateway_auth_msg = gateway_auth_msg
            self.gateway_auth_status = gateway_auth_status
            self.userid = __userid__
            self.token = f"bearer {__gatewayToken__}"
        except Exception as expp:
            self.write_log("网关认证失败, err:{}".format(expp))
            gateway_auth_status = self.gateway_auth_status
            gateway_auth_msg = self.gateway_auth_msg
            __userid__ = self.userid
            __gatewayToken__ = self.gatewayToken
            self.token = None

        return {'status': gateway_auth_status, 'msg': gateway_auth_msg, 'token': __gatewayToken__, 'userid': __userid__}

    def _get_user_info(self, token, header_userid_param):
        # 获取user-msg
        headers_userid = {
            "Authorization": 'bearer ' + token,
            "headerParam": header_userid_param
        }
        UA_URL = AU_USERID_URL.replace('__kaisarooturl__', self.root_url)
        response_userid = requests.post(url=UA_URL, headers=headers_userid)

        if response_userid.status_code // 100 == 2:
            userid_data = response_userid.json()
            user_id_num = userid_data['body']['userId']

        else:
            user_id_num = ''
        return user_id_num

    def _get_accessToken(self, data):
        # 获取登录令牌
        try:
            token_body = data["body"]["accessToken"]
            #gatewayToken = f"bearer {token_body}"
            gateway_auth_status = True
        except Exception as exp:
            print('获取登录令牌抛异常:{}'.format(exp))
            token_body = ''
            gateway_auth_status = False
        return token_body, gateway_auth_status

    def gateway_auth(self):
        return self._authentica(self.user_id, self.user_pwd)

    def init_md_api(self):
        '''
        初始化mdApi
        --login并验证行情账号
        --初始化行情Api
        '''
        try:
            data = self.gateway_auth()
            ret_msg = data['msg']
            ret_ok = data['status']
            if ret_ok:
                print('行情账号登录成功!')
                self.init_api_bool = ret_ok
            else:
                print('行情账号登录失败,error:{}!'.format(ret_msg))
        except Exception as exp:
            return 'market-ws disconnected!', exp

    def do_market_requests(self, url, reqdata):

        reqdata = self._market_req_decorate(reqdata)
        method = reqdata["method"] if ("method" in reqdata) else "POST"
        headers = reqdata["headers"] if ("headers" in reqdata) else None
        params = reqdata["params"] if ("params" in reqdata) else None
        data = reqdata["data"] if ("data" in reqdata) else None

        response = requests.request(
            method,
            url,
            headers=headers,
            params=params,
            data=json.dumps(data)
        )
        status_code = response.status_code
        resp_data = response.json()
        return status_code, resp_data

    def _market_req_decorate(self, reqdata):

        reqdata['headers'] = {
            "Authorization": self.gatewayToken,
            "Content-Type": "application/json"
        }
        return reqdata

    def query_all_symbollist(self):

        contractdata_all = []
        beginpos = 0
        count = 1000
        while True:
            symbols = self.query_symbollist(beginpos=beginpos, count=count)
            if symbols is None:
                break
            contractdata_all += symbols
            if len(symbols)<count:
                break
            beginpos += count
        if len(contractdata_all)>0:
            self.write_log("合约信息查询成功")
        else:
            self.write_log("合约信息查询失败")

        return contractdata_all

    def query_symbollist(self, beginpos=0, count=1000):

        self.req_id += 1
        data = {
            "reqtype": QUERY_CONTRACT,
            "reqid": self.req_id,
            "session": "",
            "data": {
                "marketid": HKSE_MARKET,
                "idtype": 1,
                "beginpos": beginpos,
                "count": count,
                "getquote": 1
            }
        }

        reqdata = {"data": data}
        status_code, resp_data = self.do_market_requests(url=self.QUOTE_URL, reqdata=reqdata)
        if status_code//100 == 2:
            symbols = resp_data['data']['symbol']
        else:
            symbols = None
        return symbols

    def get_market_connect_ws_status(self):

        return self._market_connect_status, self._market_connect_msg

    def on_recv(self):
        # recv
        res_data = self._open_sock.recv(1024)
        return res_data

    def on_message(self):
        # get-data
        while True:
            try:
                message = self.on_recv()
                # 把字符串转换为json数据类型
                data = kprotoc.rec_bstring_to_json(message)
                if data is None:
                    pass
                elif data['data'] is not None:
                    #  回调函数
                    self.callback_cls(data)
            except Exception as expp:
                continue

    def _on_sub(self, subscribe_code_list):
        # open-sub
        print("####### on_sub #######")
        def run(*args):

            self.req_id += 1
            sub_bstring = kprotoc.get_subscribe_bstring(subscribe_code_list, self.req_id)
            rd = self._open_sock.sendall(sub_bstring)
            time.sleep(1)
        try:
            run()
            self.subscribe_code_list = subscribe_code_list
            # 发心跳
            self.th = threading.Thread(target=self._market_ping, args=())
            self.th.start()

            # 获取数据
            th_msg = threading.Thread(target=self.on_message, args=())
            th_msg.start()

        except Exception as exp:
            print('on-open--error:{}'.format(exp))
            pass

    def on_market_packet(self, packet):
        self.on_market_data(packet)

    def on_market_data(self, data):
        # 行情数据到达>>data
        pass

    def subscribe_marketdata(self, code_list):
        """
        订阅quote和tick数据
        code_list: [{'code':'01638', 'market':2, 'sub_type':2}]
        """
        self.req_id += 1
        # 订阅
        self._on_sub(code_list)

    def unsubscribe_marketdata(self, code_list):
        """
        取消推送行情和分笔
        :param symbols:
        :return:
        """
        try:
            self.req_id += 1
            bstring = kprotoc.get_unsubscribe_bstring(code_list, self.req_id)
            self._open_sock.sendall(bstring)
        except Exception as exp:
            pass

    def _market_ping(self):
        """
        :param this_ws:
        :return:
        """
        while True:
            _now_ = datetime.datetime.now()
            try:
                with open(self.file_name, 'a') as fs:
                    mytxt = '发送心跳:{}'.format(_now_) + ', ' + '线程状态:{}'.format(self.th.is_alive()) + '.\n'
                    fs.write(mytxt)
                self.market_heartbeat()
                time.sleep(5)
            except Exception as exp:
                with open(self.file_name, 'a') as fs:
                    mytxt = '发送心跳:{}'.format(_now_) + ', 出现异常:'.format(exp) + ', ' + '线程状态:{}'.format(self.th.is_alive()) + '.\n'
                    fs.write(mytxt)
                # 断开
                self._open_sock.close()
                self.gateway_auth_status = False
                # api 重连
                self.init_open_md_api()
                # 重新订阅行情
                self.subscribe_marketdata(self.subscribe_code_list)
                time.sleep(5)

    def market_heartbeat(self):
        # 心跳
        self.req_id += 1
        ping_bstring = kprotoc.get_ping_bstring(self.req_id)
        self._open_sock.sendall(ping_bstring)

    def send_market_packet(self, packet):
        text = json.dumps(packet)
        #self._open_sock.send(text)

    def generate_req(self, reqtype: int, data: dict) -> dict:

        self.req_id += 1
        req = {
            "reqtype": reqtype,
            "reqid": self.req_id,
            "session": "",
            "data": data
        }
        return req

    def _trade_req_decorate(self, reqdata, reqtype="normal"):

        json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
        method = reqdata['method'] if 'method' in reqdata else "POST"
        headers = reqdata['headers'] if 'headers' in reqdata else {}
        data = reqdata['data'] if 'data' in reqdata else {}
        params = reqdata['params'] if 'params' in reqdata else {}

        this_timestamp = generate_timestamp()

        if (reqtype == "connect") and (method == "POST") and ('q' in reqdata['data']):
            headers = {"Content-Type": "application/json",
                       "Authorization": self.token
                       }
            params = deepcopy(reqdata['data'])
            reqdata = {'method': method,
                       'headers': headers,
                       'data': json_dumps(reqdata['data']),
                       'params': params
                       }
            return reqdata

        if reqtype == "login":
            #app_token = "bearer eyJhbGciOiJIUzI1NiJ9.eyJ1c2VyX25hbWUiOiI5NjAwNzAyMDExIiwic2NvcGUiOlsicmVhZCJdLCJhdXRoVG9rZW4iOiJ7XCJhY2Nlc3NUb2tlblZhbGlkaXR5XCI6MCxcImFwcExvZ2luVHlwZVwiOlwiY2xpZW50SWRcIixcImhlYWRlclBhcmFtVk9cIjp7XCJhcGlWZXJzaW9uXCI6XCIxLjcuMVwiLFwiY2hhbm5lbElkXCI6XCJcIixcImNvdW50cnlcIjpcIlwiLFwiZGV2aWNlSWRcIjpcIjUyRDBEM0ZCLURDNzktNDFFNS04NDQ2LThGRjAxRDc5RUY5MFwiLFwiZGV2aWNlTmFtZVwiOlwiJUU2JTlEJThFJUU2JTlGJUIxJUU2JTk2JUIwJUU3JTlBJTg0aVBob25lXCIsXCJkZXZpY2VUb2tlblwiOlwiYmE4NDk5OWU0MzRkMzBkYTZjYWQyNjg3MWE5M2NiYjY4ZDJkYzZmNDBjMzFhOWRiZmQwNmNkMzI2ZmQzOWMyZFwiLFwiaXBBZGRyZXNzXCI6XCIxMTcuMTM2LjQwLjE3OVwiLFwibGFuZ3VhZ2VcIjpcImdiXCIsXCJtYWNBZGRyZXNzXCI6XCIwMjowMDowMDowMDowMDowMFwiLFwicGhvbmVNb2RlbFwiOlwiaVBob25lIFhcIixcInBsYXRmb3JtXCI6XCJpT1NcIixcInN5c1ZlcnNpb25cIjpcIjEzLjZcIn0sXCJtb2JpbGVcIjpcIjEzNTYwNzk3MTYwXCIsXCJwaG9uZUFyZWFcIjpcIis4NlwiLFwicmVmcmVzaFRva2VuVmFsaWRpdHlcIjowLFwidGltZXN0YW1wXCI6MCxcInVzZXJJZFwiOlwiMTM2NjYzMTU0MTYwNjQwODE5NFwifSIsImV4cCI6MTYyNzE5MjIwNiwidXNlcklkIjoiTVRNMk5qWXpNVFUwTVRZd05qUXdPREU1TkE9PSIsImF1dGhvcml0aWVzIjpbIlJPTEVfQURNSU4iXSwianRpIjoiZWM1ZDQ4MzgtNWU5MC00YWMzLTk4NjAtNjQ0ODExMjBmNTYxIiwiZmlyc3QiOm51bGwsImNsaWVudF9pZCI6InRlc3RfY2xpZW50X2FwcCIsInRpbWVzdGFtcCI6MTYyNzEwNTgwNjUxN30.cJ4-pGzuosEiIv_ZqKnMPuZe8MBE4sqR9zQiwiqSruw"
            headers['Content-Type'] = "application/json"
            headers["Authorization"] = self.token
            if reqtype == "login":
                data['Authorization'] = self.token
            if self.tradeToken is not None:
                headers['X-Trade-Token'] = self.tradeToken
            if reqtype == "normal":
                if 'tradeToken' in data:
                    del data['tradeToken']
            if method == "POST":
                header_ = reqdata['data'].copy()
                if self.tradeToken is not None:
                    header_["tradeToken"] = self.tradeToken
                headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
                headers["Timestamp"] = this_timestamp

            reqdata = {"method": method,
                       "data": json_dumps(data),
                       "headers": headers,
                       "params": json_dumps(params)}
            return reqdata

        headers = {"Content-Type": "application/json",
                   "Authorization": self.token,  # self.gatewayToken,
                   "X-Trade-Token": self.tradeToken
                   }

        if method == "GET":
            if 'params' in reqdata:
                header_ = reqdata['params'].copy()
            else:
                header_ = {}
            header_["Authorization"] = self.token
            headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="GET")
            headers["Timestamp"] = this_timestamp

        if method == "POST":
            header_ = data.copy()
            if self.tradeToken is not None:
                header_["tradeToken"] = self.tradeToken
            headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
            headers["Timestamp"] = this_timestamp

        if (method == "POST") and ('q' not in data):
            q_string = json_dumps(data)
            q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
            data = {'q': q_string}

        reqdata = {"method": method,
                   "data": json_dumps(data),
                   "headers": headers,
                   "params": params}

        return reqdata

    def _sign(self, reqdata, reqtype="normal"):

        json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
        method = reqdata['method']
        headers = reqdata['headers'] if 'headers' in reqdata else {}
        data = reqdata['data'] if 'data' in reqdata else {}
        params = reqdata['params'] if 'params' in reqdata else {}

        if (reqtype == "connect") and (method == "POST") and ('q' in reqdata['data']):
            headers = {"Content-Type": "application/json",
                       "Authorization": self.token}

            params = deepcopy(reqdata['data'])
            reqdata = {'method': method,
                       'headers': headers,
                       'data': json_dumps(reqdata['data']),
                       'params': params
                       }
            return reqdata

        if reqtype == "login":

            headers['Content-Type'] = "application/json"
            headers["Authorization"] = self.token
            if reqtype == "login":
                data['Authorization'] = self.token

            if self.tradeToken is not None:
                headers['X-Trade-Token'] = self.tradeToken

            if reqtype == "normal":
                if 'tradeToken' in data:
                    del data['tradeToken']

            if method == "POST":
                timestamp_ = generate_timestamp()
                header_ = reqdata['data'].copy()
                if self.tradeToken is not None:
                    header_["tradeToken"] = self.tradeToken
                headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
                headers["Timestamp"] = timestamp_

            reqdata = {"method": method,
                       "data": json_dumps(data),
                       "headers": headers,
                       "params": json_dumps(params)}

            return reqdata

        headers = {"Content-Type": "application/json",
                   "Authorization": self.gatewayToken,
                   "X-Trade-Token": self.tradeToken
                   }

        if method == "GET":
            timestamp_ = generate_timestamp()
            header_ = reqdata['params'].copy()
            header_["Authorization"] = self.token
            headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="GET")
            headers["Timestamp"] = timestamp_

        if method == "POST":
            timestamp_ = generate_timestamp()
            header_ = data.copy()
            if self.tradeToken is not None:
                header_["tradeToken"] = self.tradeToken
            headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
            headers["Timestamp"] = timestamp_

        if (method == "POST") and ('q' not in data):
            q_string = json_dumps(data)
            q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
            data = {'q': q_string}

        reqdata = {"method": method,
                   "data": json_dumps(data),
                   "headers": headers,
                   "params": params}

        return reqdata

    def __sign(self, header_: dict = {}, request_=None, timestamp_=None):

        sign_ = None
        str_ = ""
        for key_ in sorted(header_):
            str_ += str(key_)+str(header_[key_])
        str_ += "Timestamp"+timestamp_
        sign_ = kcrypto.encrypt_md5(str_)
        return sign_

    def do_trade_requests(self, url, reqdata, reqtype="normal"):

        reqdata = self._trade_req_decorate(reqdata, reqtype)
        method = reqdata["method"] if ("method" in reqdata) else "POST"
        headers = reqdata["headers"] if ("headers" in reqdata) else None
        params = reqdata["params"] if ("params" in reqdata) else None
        data = reqdata["data"] if ("data" in reqdata) else None
        response = requests.request(
            method,
            url,
            headers=headers,
            params=params,
            data=data
        )
        status_code = response.status_code
        resp_data = response.json()
        return status_code, resp_data

    def trade_connect(self):
        """
        交易api连接
        :return: trade_connect_status
        """
        auth_data = self.gateway_auth()
        use_id = auth_data['userid']
        token = auth_data['token']
        ret_ok = auth_data['status']
        if ret_ok:
            en_password_trade = kcrypto.encrypt_aes_password(self.account_pwd, type="TRADE")
            self.en_password_trade = en_password_trade
            secret_q = kcrypto.encrypt_rsa_secretQ(crypto_rsa_url=self.TRADE_CRYPTO_RSA_URL)
            data = {
                "q": secret_q,
                "accountCode": self.account_code
            }
            url = self.REST_HOST + "/v1/account/shakeHand"
            reqdata = {"data": data}

            status_code, resp_data = self.do_trade_requests(url, reqdata, "connect")
            self.sessionId = resp_data['body']['sessionId']
            self.trade_connect_status = True

        self.trade_connect_msg = ''
        return self.trade_connect_status, self.trade_connect_msg

    def trade_auth(self):
        # 交易账户认证、登录

        headerParam = {"platform": "open", "deviceName": "",
                      "apiVersion": "1.7.1", "country": "", "channelId": "", "language": "gb",
                      "ipAddress": self.ip_address, "phoneModel": "", "sysVersion": "13.6",
                      "macAddress": get_mac_addr(), "deviceId": "",
                      "deviceToken": ""}
        data = {
            "channelType": "INTERNET",
            "accountCode": self.account_code,
            "password": self.en_password_trade,
            "secondAuthFromOther": "Y",
            "sessionId": self.sessionId
        }
        headers = {
            "headerParam": json.dumps(headerParam)
        }

        reqdata = {"data": data, "headers": headers}

        url = self.REST_HOST + "/v1/account/login"
        status_code, resp_data = self.do_trade_requests(url, reqdata, "login")
        if status_code//100 == 2:
            self.tradeToken = resp_data['body']['tradeToken']
            print('self.tradeToken:{}'.format(self.tradeToken))
            self.trade_auth_status = True
        else:
            self.tradeToken = None
            self.trade_auth_status = False
        self.trade_auth_msg = ''
        return self.trade_auth_status, self.trade_auth_msg

    def trade_connect_ws(self):
        threading.Thread(target=self._start_trade_ws, daemon=True).start()

    def get_trade_connect_ws_status(self):
        return self._ws_trade_connect_status, ""

    def _trade_ping(self, this_ws):
        """
        :param this_ws:
        :return:
        """
        pass

    def _start_trade_ws(self):

        if self._ws_trade_connect_status:
            return True

        #
        data = {
            "channelType": "INTERNET",
            "accountCode": self.account_code,
            "password": self.en_password_trade,
            "ipAddress": "",
            "secondAuthFromOther": "Y",
            "sessionId": self.sessionId,
        }

        host = self.WEBSOCKET_TRADE_HOST
        header = {"Authorization": self.token}
        # # create_connection--
        self._trade_ws = websocket.create_connection(host, header=header)

        req = self.generate_req(LOGIN, data)

        req = json.dumps(req)
        self._trade_ws.send(req)
        self._trade_ping_th = threading.Thread(target=self._trade_ping, args=(self._trade_ws,), daemon=True)
        self._trade_ping_th.start()
        self._ws_trade_connect_status = True

        while True:
            text = self._trade_ws.recv()
            self.on_trade_packet(text)

    def on_trade_packet(self, packet):

        if len(packet) == 0:
            return
        data = json.loads(packet)
        if data.get('reqtype', 0) == 2:
            req_data = {"ts": data['data']["ts"]}
            req_data = self.generate_req(PONG, req_data)
            self._trade_ws.send(json.dumps(req_data))
        else:
            data = json.loads(packet)
            self.on_trade_data(packet)

    def send_order(self, bsFlag='B', price=550, qty=600, code='00700'):

        return self.place_order(bsFlag=bsFlag, price=price, qty=qty, code=code)

    def place_order(self, bsFlag='B', price=550, qty=600, code='00700'):

        data = {'channelType': "I",
                'exchangeCode': 'HKEX',
                'accountCode': self.account_code,
                'productCode': code,
                'price': price, 'qty': qty,
                'bsFlag': bsFlag, 'orderType': 'L',
                'tradeToken': self.tradeToken}

        reqdata = {"data": data,
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/order/orders/place"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    def cancel_order(self, sys_orderid):

        data = {
            "channelType": "I",
            "accountCode": self.account_code,
            "orderID": sys_orderid,
            "tradeToken": self.tradeToken,
        }
        reqdata = {"data": data,
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/order/orders/cancel"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    def query_portfolio(self):

        reqdata = {"method":"GET",
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/account/accounts/portfolio"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    def query_balance(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/account/accounts/balance"
        status_code, resp_data = self.do_trade_requests(url, reqdata)
        resp_data = resp_data.json()
        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    def query_position(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/account/accounts/position"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    def query_order(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/order/orders"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

if __name__=="__main__":

    print("Hello, Kaisa.")




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg_comm_cust.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg_comm_cust.proto',
  package='MsgCommCust',
  syntax='proto3',
  serialized_options=b'\n\034com.kaisaglobal.proto.common\200\001\000\210\001\001\220\001\000',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13msg_comm_cust.proto\x12\x0bMsgCommCust\"e\n\x08\x43ustHead\x12\x10\n\x08packFlag\x18\x01 \x01(\r\x12\x11\n\tiCompress\x18\x02 \x01(\r\x12\x10\n\x08iVersion\x18\x03 \x01(\r\x12\x10\n\x08iReserve\x18\x04 \x01(\r\x12\x10\n\x08iCheckId\x18\x05 \x01(\rB\'\n\x1c\x63om.kaisaglobal.proto.common\x80\x01\x00\x88\x01\x01\x90\x01\x00\x62\x06proto3'
)




_CUSTHEAD = _descriptor.Descriptor(
  name='CustHead',
  full_name='MsgCommCust.CustHead',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='packFlag', full_name='MsgCommCust.CustHead.packFlag', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iCompress', full_name='MsgCommCust.CustHead.iCompress', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iVersion', full_name='MsgCommCust.CustHead.iVersion', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iReserve', full_name='MsgCommCust.CustHead.iReserve', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iCheckId', full_name='MsgCommCust.CustHead.iCheckId', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=137,
)

DESCRIPTOR.message_types_by_name['CustHead'] = _CUSTHEAD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CustHead = _reflection.GeneratedProtocolMessageType('CustHead', (_message.Message,), {
  'DESCRIPTOR' : _CUSTHEAD,
  '__module__' : 'msg_comm_cust_pb2'
  # @@protoc_insertion_point(class_scope:MsgCommCust.CustHead)
  })
_sym_db.RegisterMessage(CustHead)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)




# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# Technical index data
# ===================================================================
import sys
from os.path import dirname, abspath
import requests, datetime
import pandas as pd
import numpy as np
import json
from common.com import *
from pub_cls_com import *
'''Date: 2021.03.16'''

def _handle_ex_ret(code_list=['00700'], queryType=None, start_date=None, end_date=None, market=2, unit='1d', page=1, fre=2):

    # 查询最长周期: 1年时间
    if unit != '1d':
        return TECH_ONLY_DAY_DATA_ERROR
    if market == 3 or market == 1:
        return ONLY_HK_MARKET_ERROR
    if start_date is None and end_date is None:
        return PUB_DATENONE_ERROR
    if start_date is not None and end_date is None:
        end_date = start_date
    if start_date is None and end_date is not None:
        start_date = end_date
    if start_date is not None and end_date is not None:
        if start_date > end_date:
            return PUB_SMALLENDDATE_ERROR
        s_dt = datetime.datetime.strptime(start_date, '%Y-%m-%d')
        e_dt = datetime.datetime.strptime(end_date, '%Y-%m-%d')

        day_count = (e_dt - s_dt).days
        if day_count > 365 * 1:
            return TECH_ONLY_1_YEAR_DATA_ERROR
    body = {
        "codeList": code_list,
        "market": market,
        "queryType": queryType,
        "startDate": start_date,
        "endDate": end_date,
        "unit": unit,
        "ifRehabili": fre,
        "current": page,
        "size": 100
    }
    update_url = lambda url: url.replace("__kaisarooturl__", KAISA_ROOT_URL)
    df_ = total_handle_req(body=body, URL=update_url(BE_TECH_POST_URL))

    return df_

###### 基础类技术面指标
def get_sar_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
        获取SAR技术指标数据
        Args:
            code_list: 合约列表
            queryType:数据类型
            market: 市场类型：1 CH/ 2 HK/ 3 USA
            unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
            start_date: 开始日期
            end_date: 结束日期
            fre:是否复权  1 不复权/2 前复权/3 后复权
        Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'Trend', 'WhetherTurn', 'AF', 'SAR1', 'SAR2', 'TradingDay']
    data_type = 'tradition'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_obv_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取OBV技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'OBV', 'TradingDay']
    data_type = 'tradition'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_boll_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取BOLL技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'MID', 'UPPERValue', 'LOWERValue', 'TradingDay']
    data_type = 'tradition'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    try:
        __.columns = ['SecuCode', 'MidLine', 'UpLine', 'DownLine', 'TradingDay']
    except:
        return __
    return __

def get_wr_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取WR技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'WR6', 'WR10', 'WR13', 'WR34', 'WR89', 'TradingDay']
    data_type = 'tradition'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_rsi_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取RSI技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'RSI', 'RS', 'TradingDay']
    data_type = 'tradition'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_kdj_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取kdj技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'HighestInNine', 'LowestInNine', 'RSV', 'Kvalue', 'Dvalue', 'Jvalue', 'TradingDay']
    data_type = 'tradition'
    # f_data_ = _handle_ex_ret(code_list, data_type, start_date, end_date, market, unit, 1, fre)
    # __ = _get__(f_data_, factor_names, data_type, code_list, start_date, end_date, market, unit, fre)
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_macd_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取macd技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权  1 不复权/2 前复权/3 后复权
    Returns: 返回因子数据
    '''
    factor_names = ['SecuCode', 'EMA12', 'EMA26', 'DIF', 'DEA', 'MACD', 'TradingDay']
    data_type = 'tradition'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def _get__(f_data_, factor_names, data_type, code_list, start_date, end_date, market, unit, fre):
    '''
    Args:
        factor_names:  所选因子字段
        f_data_: 后台api返回的数据
        data_type: 技术指标类型 tradition 传统技术指标；derived 衍生技术指标
        code_list: 合约列表
        start_date: 开始日期
        end_date: 结束日期
        market: 市场类型 1 CH/ 2 HK/ 3 USA
        unit: 频率 默认 '1d' 日级别数据
        fre: 是否复权  1 不复权/2 前复权/3 后复权
    Returns:
    '''

    pageSize = f_data_['totalPage']
    page1_data = f_data_['result']
    res_df = pd.DataFrame(page1_data)
    if pageSize > 1:
        res_df = _get_other_data(res_df, factor_names, pageSize, data_type, code_list, start_date, end_date, market, unit, fre)
    __ = res_df[factor_names]
    __ = __.reset_index(drop=True)
    return __

def _get_other_data(page1_data, factor_names, page_size, data_type, code_list, start_date, end_date, market, unit, fre=2):
    # 获取剩下页码的数据
    df1_ = page1_data.copy()
    for pageC in range(2, page_size + 1):
        profit_data_next = _handle_ex_ret(code_list, data_type, start_date, end_date, market, unit, pageC, fre)

        page_n_data = profit_data_next['result']
        df1_ = pd.concat([df1_, pd.DataFrame(page_n_data)], axis=0)
    return df1_

##### 衍生类技术面指标
def get_hbeta_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Hbeta技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'hbeta', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_halpha_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Halpha技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'halpha', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_hsigma_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Hsigma技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'hsigma', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_dmsens_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Dmsens技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'dmsens', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_oilsen_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Oilsen技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'oilsen', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_dsbeta_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Dsbeta技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'dsbeta', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_cmra_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Cmra技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'cmra', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_stom_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Stom技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'stom', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_stoq_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Stoq技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'stoq', 'trading_day']
    data_type = 'derived'
    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_stoa_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取Stoa技术指标数据
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'stoa', 'trading_day']
    data_type = 'derived'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_dastd_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取中期波动率技术指标数据  Dastd
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回macd的因子数据
    '''
    factor_names = ['secu_code', 'dastd', 'trading_day']
    data_type = 'derived'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_season_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取周期性技术指标数据 Season
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回因子数据
    '''
    factor_names = ['secu_code', 'season', 'trading_day']
    data_type = 'derived'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_rstr_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取中期动量技术指标数据  Rstr
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回因子数据
    '''
    factor_names = ['secu_code', 'rstr', 'trading_day']
    data_type = 'derived'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def get_strev_factor_data(code_list=['00700'], market=2, unit='1d', start_date=None, end_date=None, fre=2):
    '''
    获取短期反转技术指标数据  strev
    Args:
        code_list: 合约列表
        queryType:数据类型
        market: 市场类型：1 CH/ 2 HK/ 3 USA
        unit: 频率 默认为 '1d', 将支持如下周期: '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
        start_date: 开始日期
        end_date: 结束日期
        fre:是否复权 默认不复权
    Returns: 返回因子数据
    '''
    factor_names = ['secu_code', 'strev', 'trading_day']
    data_type = 'derived'

    __ = total_cls_handle(code_list, factor_names, data_type, start_date, end_date, market, unit, fre)
    return __

def total_cls_handle(code_list, factor_names, data_type=None, start_date=None, end_date=None, market=None, unit=None, fre=None):
    # 统一处理返回值
    try:
        code_list = list(set(code_list))
        f_data_ = _handle_ex_ret(code_list, data_type, start_date, end_date, market, unit, 1, fre)
        if t_typeof(f_data_) == 'str' or PUB_HELLO in f_data_ or PUB_SOR in f_data_:
            return f_data_
        if len(f_data_['result']) == 0:
            return PUB_WRONG_DATE_AROUND_ERROR
        __ = _get__(f_data_, factor_names, data_type, code_list, start_date, end_date, market, unit, fre)
    except Exception as xp:
        __ = xp
    return __


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# Quote data
# ===================================================================

import sys
from os.path import dirname, abspath
import requests, datetime
import pandas as pd
import numpy as np
import json, time
from com import *
from pub_cls_com import *

def to_get_all_data(body, URL, all_pages):
    # 统一获取数据的接口
    __ = pd.DataFrame()
    for p in range(1, all_pages + 1):
        body['pageNum'] = p

        _values_ = total_handle_hist_data_req(body, URL=URL)
        _t_df_ = pd.DataFrame(_values_)
        __ = pd.concat([__, _t_df_])

    return __


def __handle_hist_data_ex_ret(code_list=['00700'],
                              start_date=None,
                              end_date=None,
                              unit='1d',
                              market=100,
                              page=1,
                              fre='',
                              data_type=None):
    # 获取历史历史K线数据
    try:
        update_url = lambda url: url.replace("__kaisarooturl__", KAISA_ROOT_URL)
        if data_type is None or data_type not in ORI_DATA_TYPE_LIST:
            return DATETYPE_ERROR

        if market not in [2, 3, 1]:
            return ONLY_HK_MARKET_ERROR
        if start_date is None and end_date is None:
            return PUB_DATENONE_ERROR
        if start_date is not None and end_date is None:
            end_date = start_date
        if start_date is None and end_date is not None:

            start_date = end_date
        if start_date is not None and end_date is not None:
            if start_date > end_date:
                return PUB_SMALLENDDATE_ERROR

        if len(code_list) == 0:
            return LOSSCODES_ERROR
        code_market_types = HistQuote_MARKET_DICT[market]

        s_dt = datetime.datetime.strptime(start_date, '%Y-%m-%d')
        e_dt = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if data_type == KLINE:

            if len(code_list) == 1:
                day_count = (e_dt - s_dt).days
                if day_count > 365 * 10:
                    return OUTDATEFOR10_ERROR

            if len(code_list) > 1:
                day_count = (e_dt - s_dt).days
                if day_count > 30 * 1:
                    return OUTDATEFOR1MON_ERROR
            code_list = [code.upper() for code in code_list]

            if unit not in K_QUOTE_TYPE_DICT.keys():
                return WRIONGUNIT_ERROR

            if fre not in FRE_PRE_DICT.keys():
                return FRE_ERROR
            fre = FRE_PRE_DICT[fre]
            body = {
                "codes": code_list,
                "market": code_market_types,
                "quoteType": K_QUOTE_TYPE_DICT[unit],
                "startDate": start_date,
                "endDate": end_date,
                "adjFactorType": fre,
                "pageNum": page,
                "pageSize": 1000
            }

            URL = update_url(HK_STOCK_KLINE_POST_URL)   # HK_STOCK_KLINE_POST_URL
            _pages_ = handle_hist_data_req_pages(body=body, URL=URL)
            ori_cols = KLINE_ORI_COLS
            res_cols = KLINE_RES_COLS

        else:
            body = {
                "codes": code_list,
                "market": code_market_types,
                "startDate": start_date,
                "endDate": end_date,
                "pageNum": page,
                "pageSize": 1000
            }
            if data_type == TICKER:
                URL = update_url(HK_STOCK_TICK_POST_URL)  # HK_STOCK_TICK_POST_URL
                ori_cols = TICKER_ORI_COLS
                res_cols = TICKER_RES_COLS
            elif data_type == BROKER:
                URL = update_url(HK_STOCK_BROKER_URL)
                ori_cols = BROKER_ORI_COLS
                res_cols = BROKER_RES_COLS
                print('url:{}'.format(URL))
                print('ori_cols:{}'.format(ori_cols))
                print('res_cols:{}'.format(res_cols))

            elif data_type == LEVEL10:
                URL = update_url(HK_STOCK_LEVEL2_10_ORDER_URL)
                ori_cols = LEVEL10_ORI_COLS
                res_cols = LEVEL10_RES_COLS
            else:
                URL = ''
                ori_cols = []
                res_cols = []
            day_count = (e_dt - s_dt).days
            if day_count > 1:
                return '{},{}数据, {}'.format(PUB_HELLO, data_type, ONEDAYUPDATE_ERROR)

        _pages_ = handle_hist_data_req_pages(body=body, URL=URL)

        __ = to_get_all_data(body, URL, _pages_)
        __ = __.sort_values(by=['assetId'])
        __ = __[ori_cols]
        if data_type == KLINE:
            __[KLINE_TIMESTAMP] = [change_timestamp(dtt) for dtt in __[KLINE_TIMESTAMP]]

            __.sort_values(by=[KLINE_TIMESTAMP], inplace=True)
        __.columns = res_cols

        return __
    except Exception as xp:
        return '{}, 您填入的参数有误:{}, 请检查后重新请求!'.format(PUB_HELLO, xp)




def get_history_kline_data(code_list=['00700'], start_date=None, end_date=None, unit='', market=2, fre='pre'):
    '''
    # 获取合约代码历史区间K线数据
    '''

    __ = __handle_hist_data_ex_ret(code_list,
                                   start_date,
                                   end_date,
                                   unit=unit,
                                   market=market,
                                   page=1,
                                   fre=fre,
                                   data_type='Kline')

    return __

def get_history_ticker_data(code_list=['00700'], start_date=None, end_date=None, market=2):
    '''
    # 获取历史逐笔数据
    '''
    __ = __handle_hist_data_ex_ret(code_list, start_date, end_date, market=market, data_type='ticker')

    return __

def get_history_broker_data(code_list=['00700'], start_date=None, end_date=None, market=2):
    '''
    # 获取历史经纪商数据
    '''
    __ = __handle_hist_data_ex_ret(code_list, start_date, end_date, market=market, data_type='broker')

    return __

def get_history_level10_data(code_list=['00700'], start_date=None, end_date=None, market=2):
    '''
    # 获取历史Level10档盘口数据
    '''
    __ = __handle_hist_data_ex_ret(code_list, start_date, end_date, market=market, data_type='level10')

    return __

if __name__ == '__main__':
    pass




